#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <ctype.h>

#include "global.h"
#include "treefile.h"

#define INSERT 1
#define DELETE 2

char treeErrorMsg[256];
int treeError;

/*CS:*/
int indelRoot;
char *indelRootSequence;
short *indelRootInvariable;
/**/

TNode *avail=NULL;
long usedAvail=0;
long usedMalloc=0;

/* prototypes */

TNode *NewNode(TTree *tree);
void InitTree(TTree *tree);
void CheckCapacity(TTree *tree, int required);
void DisposeNode(TNode *aNode);
void DisposeTreeNodes(TTree *tree);
void FreeNodes(void);

char ReadToNextChar(FILE *fv);
void ReadUntil(FILE *fv, char stopChar, char *what);
TNode *ReadTip(FILE *fv, char ch, TTree *tree, int numNames, char **names);
TNode *ReadNode(FILE *fv, TTree *tree, int numNames, char **names, int detectPolytomies);
TNode *ReadBranch(FILE *fv, TTree *tree, int numNames, char **names);
void WriteNode(FILE *fv, TTree *tree, TNode *node);

/*CS:*/
void SetLengthDistribution(TTree *tree, char *file,int action);
int Read_Indel_Root(char *filename);

/* functions */

/*----------*/
TNode *NewNode(TTree *tree)
{
	TNode *node;

	if ( avail!=NULL ) {
		node=avail;
		avail=avail->next;
		
		usedAvail++;
	} else {
		if ( (node=malloc(sizeof(TNode)))==NULL ) {
			strcpy(treeErrorMsg, "Out of memory");
			return NULL;
		}
		usedMalloc++;
	}
	node->branch0=node->branch1=node->branch2=NULL;
	node->length0=node->length1=node->length2=0.0;
	node->param=0.0;
	node->tipNo=-1;
	node->mytipNo=-1;
	node->indels=0;
	node->anc=NULL;
	node->sequence = NULL;
	
	node->next=tree->nodeList;	
	tree->nodeList=node;
	
	tree->numNodes++;

	return node;
} /* NewNode */


/*----------*/
void InitTree(TTree *tree)
{	
	tree->root=NULL;
	tree->nodeList=NULL;
	tree->numNodes=0;
	tree->numTips=0;
	tree->totalLength=0.0;
	tree->rooted=0;
	tree->lengths=-1;

	tree->do_indels=0;		/*CS: Initialized tree to no indels.*/
	tree->maxIndel=0;
	tree->P_ins_=0;
	tree->P_del_=0;
	tree->insert_lengthDistribution=(double*)malloc(MAX_INDEL_SIZE*sizeof(double));
	tree->delete_lengthDistribution=(double*)malloc(MAX_INDEL_SIZE*sizeof(double));
} /* InitTree */


/*----------*/
TTree *NewTree()
{
	TTree *tree;
	
	if ( (tree=(TTree *)malloc(sizeof(TTree)))==NULL ) {
		strcpy(treeErrorMsg, "Out of memory creating tree.");
		return NULL;
	}
	tree->capacity=0;
	CheckCapacity(tree, 1000);
	
	InitTree(tree);
	
	return tree;
} /* NewTree */


/*----------*/
void CheckCapacity(TTree *tree, int required)
{
	int i;
	int newCapacity = tree->capacity;
	char **newNames;
	TNode **newTips;
	
	while (newCapacity < required) {
		newCapacity += 1000;
	}
	
	newNames = (char**)CAllocMem(sizeof(char*)*newCapacity, "newNames", "CheckCapacity", 0);
	newTips = (TNode**)CAllocMem(sizeof(TNode*)*newCapacity, "newTips", "CheckCapacity", 0);
	
	for (i = 0; i < tree->capacity; i++) {
		newNames[i] = tree->names[i];
		newTips[i] = tree->tips[i];
	}
	
	for (i = tree->capacity; i < newCapacity; i++) {
		newNames[i] = NULL;
		newTips[i] = NULL;
	}
	
	free(tree->names);
	free(tree->tips);
	
	tree->names = newNames;
	tree->tips = newTips;
	tree->capacity = newCapacity;
} /* CheckCapacity */


/*----------*/
void DisposeNode(TNode *aNode)
{
	aNode->next=avail;
	avail=aNode;
} /* DisposeNode */
			

/*----------*/
void DisposeTreeNodes(TTree *tree)
{
	TNode *P, *O;
	
	if ( tree ) {
		P=tree->nodeList;
		while (P!=NULL) {
			O=P;
			P=P->next;
			DisposeNode(O);
		}
		tree->nodeList=NULL;
	}
} /* DisposeTreeNodes */


/*----------*/
void DisposeTree(TTree *tree)
{
	if (tree) {
		DisposeTreeNodes(tree);
		InitTree(tree);
	}
} /* DisposeTree */


/*----------*/
void FreeNodes(void)
{
	TNode *P, *O;
	
	P=avail;
	while (P!=NULL) {
		O=P;
		P=P->next;
		free(O);
	}
} /* FreeNodes */


/*----------*/
void FreeTree(TTree *tree)
{
	if (tree) {
		DisposeTreeNodes(tree);
		free(tree);
	}
	FreeNodes();
} /* FreeTree */


/*----------*/
void WriteAvailInfo()
{
	TNode *P;
	int count;
	
	count=0;
	P=avail;
	while (P!=NULL) {
		P=P->next;
		count++;	
	}

	fprintf(stderr, "Avail: %d nodes - availed: %ld, malloced: %ld\n", count, usedAvail, usedMalloc);
} /* WriteAvailInfo */


int CountTrees(FILE *fv)
{
	int n;
		
	if (fv==NULL)
		return 0;
		
	n=0;
	while (!feof(fv)) {
		if (fgetc(fv)==';')
			n++;
	}
	rewind(fv);
	
	return n;
}

char ReadToNextChar(FILE *fv)
{
	char ch;
	
	ch=fgetc(fv);
	while (!feof(fv) && isspace(ch)) ch=fgetc(fv);

	return ch;
}


void ReadUntil(FILE *fv, char stopChar, char *what)
{
	char ch;
	
	ch=fgetc(fv);
	while (!feof(fv) && ch!=stopChar && 
			ch!='(' && ch!=',' && ch!=':' && ch!=')' && ch!=';') 
		ch=fgetc(fv);

	if (feof(fv) || ch!=stopChar) {
		sprintf(treeErrorMsg, "%s missing", what);
		treeError=1;
	}
}


TNode *ReadTip(FILE *fv, char ch, TTree *tree, int numNames, char **names)
{
	int i;
	char *P;
	char name[256];
	
	TNode *node;
	
	node=NewNode(tree);
	i=0;
	
	P=name;
	while (!feof(fv) && ch!=':' && ch!=',' && ch!=')' && i<MAX_NAME_LEN) {
		if (!isspace(ch)) {
			*P=ch;
			i++;P++;
		}
		ch=fgetc(fv);
	}
	*P='\0';

	CheckCapacity(tree, tree->numTips);
	
	if (numNames == 0) {
		node->tipNo=tree->numTips;
		if (tree->names[node->tipNo]==NULL) {
			if ( (tree->names[node->tipNo]=(char *)malloc(MAX_NAME_LEN+1))==NULL ) {
				strcpy(treeErrorMsg, "Out of memory creating name.");
				return NULL;
			}
		}
		strcpy(tree->names[node->tipNo], name);
	} else {
	/* we already have some names so just look it up...*/
		i = 0;
		while (i < numNames && strcmp(name, names[i]) != 0)
			i++;

		if (i == numNames) {
			sprintf(treeErrorMsg, "Taxon names in trees for different partitions do not match.");
			return NULL;
		}
		
		node->tipNo=i;
	}
	
	tree->tips[node->tipNo]=node;
	tree->numTips++;

	while (!feof(fv) && ch!=':' && ch!=',' && ch!=')') 
		ch=fgetc(fv);

	if (feof(fv)) {
		sprintf(treeErrorMsg, "Unexpected end of file");
		return NULL;
	}
	ungetc(ch, fv);
	
	return node;
}


TNode *ReadNode(FILE *fv, TTree *tree, int numNames, char **names, int detectPolytomies)
{
	TNode *node, *node2;
	char ch;
	
	if ((node=NewNode(tree))==NULL)
		return NULL;

	if ((node2=ReadBranch(fv, tree, numNames, names))==NULL)
		return NULL;
	node->branch1=node2;
	node2->branch0=node;
	node->length1=node2->length0;
	ReadUntil(fv, ',', "Comma");
	if (treeError)
		return NULL;
	
	if ((node2=ReadBranch(fv, tree, numNames, names))==NULL)
		return NULL;
	node->branch2=node2;
	node2->branch0=node;
	node->length2=node2->length0;
	
	ch=fgetc(fv);
	while (!feof(fv) && ch!=':' && ch!=',' && ch!=')' && ch!=';') 
		ch=fgetc(fv);
		
	if (detectPolytomies && ch==',') {
		fprintf(stderr, "This tree contains nodes which aren't bifurcations. Resolve the node\n");
		fprintf(stderr, "with zero branch lengths to obtain correct results. This can be done\n");
		fprintf(stderr, "with a program called TreeEdit: http://evolve.zoo.ox.ac.uk/software/TreeEdit\n");
		exit(0);
	}

	if (feof(fv)) {
		sprintf(treeErrorMsg, "Unexpected end of file");
		return NULL;
	}
	ungetc(ch, fv);
	
	return node;
}


TNode *ReadBranch(FILE *fv, TTree *tree, int numNames, char **names)
{
	char ch;
	double len, param=0.0;
	TNode *node;
	
	ch=ReadToNextChar(fv);
	if (ch=='(') {	/* is a node */
		node=ReadNode(fv, tree, numNames, names, 1);
		ReadUntil(fv, ')', "Closing bracket");
		if (treeError)
			return NULL;
	} else {		/* is a tip */
		node=ReadTip(fv, ch, tree, numNames, names);
	}
	
	ch=ReadToNextChar(fv);
	if (ch==':') {
		if (tree->lengths==0) {
			sprintf(treeErrorMsg, "Some branches don't have branch lengths");
			return NULL;
		} else 
			tree->lengths=1;
			
		if (fscanf(fv, "%lf", &len)!=1) {
			sprintf(treeErrorMsg, "Unable to read branch length");
			return NULL;
		}

		ch=ReadToNextChar(fv);
		if (ch=='[') {
			if (fscanf(fv, "%lf", &param)!=1) {
				sprintf(treeErrorMsg, "Unable to read branch parameter");
				return NULL;
			}
			ReadUntil(fv, ']', "Close square bracket");
		} else
			ungetc(ch, fv);
	} else {
		if (tree->lengths==1) {
			sprintf(treeErrorMsg, "Some branches don't have branch lengths");
			return NULL;
		} else 
			tree->lengths=0;
	
		len=0.0;
		ungetc(ch, fv);
	}
	node->length0=len;
	node->param=param;
	
	return node;
}	


void ReadTree(FILE *fv, TTree *tree, int treeNum, int numNames, char **names, 
				int *outNumSites, double *outRelRate)
{
    char ch;
    TNode *P;

    /*CS: save some space for fscanf() calls*/
    double *indelRates=(double*)malloc(sizeof(double));
    int *R=(int*)malloc(sizeof(int));
    char indelRootFile[128];
    char *S;
    char filename[128];
    int i;
    /**/


    /*CS:*/indelRoot=0;

    treeError=0;
    tree->numNodes=0;
    tree->numTips=0;
    tree->rooted=1;
    tree->lengths=-1;

    (*outRelRate) = 1.0;

    ch=fgetc(fv);
    while (!feof(fv) && ch!='(' && ch!='[' /*CS:*/ && ch!='{') ch=fgetc(fv);

    if (ch == '[') {
	ch=fgetc(fv);
	//fprintf(stderr,"character: %c\n",ch);
	/*CS: This if-else statement is added to allow for a direct root sequence + indel/invariable site specs */
	if(ch == ':') {		/* Then we are reading in a filename with a root seq & invariable sites. */
	    /* Read the treefile, set outNumSites to be equal to the length of the sequence. */
	    indelRoot=1;
	    invariableSites=1;
	    ch=fgetc(fv);
	    S=indelRootFile;
	    i=0;
	    while(!feof(fv) && ch!=']' && ch!=',' && i<MAX_NAME_LEN) {
		if (!isspace(ch)) {
		    *S=ch;
		    i++;S++;
		} else {
		    fprintf(stderr,"Length distribution filename should not have a space.\n");
		    exit(0);
		}
		ch=fgetc(fv);
	    }
	    *S='\0';
	    *outNumSites=Read_Indel_Root(indelRootFile);
	    ch=fgetc(fv);
	    while (!feof(fv) && ch!=',' && ch!='(' && ch!='{') ch=fgetc(fv); 
	} else {
	    ungetc(ch,fv);
	    if (fscanf(fv, "%d", outNumSites)!=1) {
	    	sprintf(treeErrorMsg, "Unable to read partition length");
	    	exit(0);
	    }

	    ch=fgetc(fv);
	    while (!feof(fv) && ch!=',' && ch!='(' /*CS:*/ && ch!='{') ch=fgetc(fv);

	    if (ch == ',') {
	    	if (fscanf(fv, "%lf", outRelRate)!=1) {
		    sprintf(treeErrorMsg, "Unable to read partition relative rate");
		    exit(0);
	        }

	        ch=fgetc(fv);
	        while (!feof(fv) && ch!='(' /*CS:*/ && ch!='{') ch=fgetc(fv);
	    }
	}
    }

    /*CS: Read in indel options*/
    if (ch == '{') {
	/* Set the default values */
	filename[0]='\0';
	tree->P_ins_=0;
	tree->P_del_=0;
	tree->maxIndel=20;
	indel_flag=1;

	// Set indel option to yes.
	tree->do_indels=1;
	if (fscanf(fv, "%d", R)!=1) { 
	    sprintf(treeErrorMsg, "Unable to read maximum indel size");
	    fprintf(stderr,"tree->maxIndel: %d, R=%d, but in error.\n",tree->maxIndel,*R);
	    exit(0);
	}

	if(*R > MAX_INDEL_SIZE) {
	    fprintf(stderr, "The maximum indel size allowed is %d\n",MAX_INDEL_SIZE);
	    exit(0);
	}
	tree->maxIndel=*R;
	//fprintf(stderr,"tree->maxIndel: %d\n",tree->maxIndel);

	ch=fgetc(fv);
	while (!feof(fv) && ch!=',' && ch!='(') ch=fgetc(fv);

	if (ch == ',') {
	    if (fscanf(fv, "%lf", indelRates)!=1) {
		sprintf(treeErrorMsg, "Unable to read the indel model");
		exit(0);
	    }
	    if (*indelRates < 0) {
		sprintf(treeErrorMsg, "Indel rate must be a positive value");
		exit(0);
	    } else if (*indelRates > 100) {
		sprintf(treeErrorMsg, "Indel rate cannot be greater than 100 %%");
		exit(0);
	    }

	    /* Use Chang and Benner's model if indelRates==0. */
	    /* With this, can check for if(!tree->P_ins_) to make the correct model assumption. */
	    tree->P_ins_=*indelRates;
	    //fprintf(stderr,"tree->P_ins_: %lf\n",tree->P_ins_);

	    ch=fgetc(fv);
	    while (!feof(fv) && ch!=',' && ch!='(' && ch!='/') ch=fgetc(fv);  

	    if (ch == '/') {
		if (fscanf(fv, "%lf", indelRates)!=1) {
		    sprintf(treeErrorMsg, "Unable to read the indel model");
		    exit(0);
		}
	        ch=fgetc(fv);
		tree->P_del_=*indelRates;
	    } else { tree->P_del_=tree->P_ins_; }
	    //fprintf(stderr,"tree->P_del_: %lf\n",tree->P_del_);
		if (tree->P_del_ == 0 && tree->P_ins_ == 0) {
			SetLengthDistribution(tree, filename, INSERT);
			SetLengthDistribution(tree, filename, DELETE);
		}

	    while (!feof(fv) && ch!=',' && ch!='(' && ch!='/') ch=fgetc(fv);

	    /* Read in a length distribution file */
	    if (ch == ',') {
		ch=fgetc(fv);
		S=filename;
		i=0;
		while(!feof(fv) && ch!='}' && ch!='/' && i<MAX_NAME_LEN) {
		    if (!isspace(ch)) {
			*S=ch;
			i++;S++;
		    } else {
			fprintf(stderr,"Length distribution filename should not have a space.\n");
			exit(0);
		    }
		    ch=fgetc(fv);
	 	}
		*S='\0';
	        SetLengthDistribution(tree,filename,INSERT);
		if(ch == '/') {
		    ch=fgetc(fv);
		    i=0;
		    S=filename;
		    while(!feof(fv) && ch!='}' && ch!='/' && i<MAX_NAME_LEN) {
		    	if (!isspace(ch)) {
			    *S=ch;
			    i++;S++;
		    	} else {
			    fprintf(stderr,"Length distribution filename should not have a space.\n");
			    exit(0);
		    	}
		        ch=fgetc(fv);
	 	    }
		    *S='\0';
	            SetLengthDistribution(tree,filename,DELETE);
		} else {
		    ch=fgetc(fv);
		    SetLengthDistribution(tree,filename,DELETE);
		}
		while (!feof(fv) && ch!=',' && ch!='(') ch=fgetc(fv); 
	    }
	}
	while (!feof(fv) && ch!=',' && ch!='(') ch=fgetc(fv); 
    } /* This is now the end of the indel reading. */
    /*endCS*/

    if (ch!='(' || (tree->root=ReadNode(fv, tree, numNames, names, 0))==NULL) {
	fprintf(stderr, "Error reading tree number %d: %s.\n", treeNum, treeErrorMsg);
	exit(0);
    }

    ch=fgetc(fv);
    while (!feof(fv) && ch!=',' && ch!=')' && ch!=';') ch=fgetc(fv);

    if (ch==',') {		/* the tree is unrooted */
	tree->rooted=0;
	if ((tree->root->branch0=ReadBranch(fv, tree, numNames, names))==NULL) {
	    fprintf(stderr, "Error reading tree number %d: %s.\n", treeNum, treeErrorMsg);
	    exit(0);
	}
	tree->root->branch0->branch0=tree->root;
	tree->root->length0=tree->root->branch0->length0;
    }

    tree->totalLength=0.0;

    if (tree->rooted) {
	P=tree->root;
	while (P!=NULL) {
	    tree->totalLength+=P->length0;
	    P=P->branch1;
	}
    }
}


/*CS: This sets the external variables for the root sequence read. */
int Read_Indel_Root(char *filename) {
    FILE *ldFile;
    int seq_len;
    char ch;
    int i;

    if ( (ldFile=fopen(filename, "rt"))==NULL ) {
	fprintf(stderr, "Error opening length distribution file: '%s'\n", filename);
	exit(0); 
    }
    seq_len=0;
    if (fscanf(ldFile,"%d",&seq_len)!=1) {
	fprintf(stderr,"Unable to read indel root sequence file: '%s'\n",filename);
	exit(0);
    }
    //fprintf(stderr,"Input sequence is %d amino acids long.\n",seq_len);
    indelRootSequence=(char*)malloc((seq_len+1)*sizeof(char));
    indelRootInvariable=(short*)malloc((seq_len+1)*sizeof(short));

    while(ch!='0' && ch!='1' && ch!='2') ch=fgetc(ldFile);

    i=0;
    //fprintf(stderr,"indelRootInvariable and indelRootSequence:\n");
    while(!feof(ldFile) && i<seq_len) {
	indelRootInvariable[i]=ch-48;
	//fprintf(stderr,"%d",indelRootInvariable[i]);
	i++;
	ch=fgetc(ldFile);
    }
    //fprintf(stderr,"\n");

    while(((int)ch < 64 || (int)ch > 91) && ((int)ch < 96 || (int)ch > 123)) ch=fgetc(ldFile);

    i=0;
    while(!feof(ldFile) && i<seq_len) {
	indelRootSequence[i]=ch;
	//fprintf(stderr,"%c",indelRootSequence[i]);
	i++;
	ch=fgetc(ldFile);
    }
    //fprintf(stderr,"\n");
    fclose(ldFile);

    return seq_len;
}


/*CS: This function opens the filename where the user-defined length distribution resides. */
void SetLengthDistribution(TTree *tree, char *filename,int action) {
    FILE *ldFile;
    int i;
    double sum=0; 
    double P;
    char ch;

fprintf(stderr, "ins = %f, del = %f\n", tree->P_ins_, tree->P_del_);

    if(tree->P_ins_ == 0 && tree->P_del_ == 0) {	/* Then Benner & Chang's indel model, set B&C length distribution. */
	for (i=1; i<=tree->maxIndel; i++) { 
            tree->insert_lengthDistribution[i] = 2628.0 * pow(i,-1.821); 
            tree->delete_lengthDistribution[i] = 2628.0 * pow(i,-1.821); 

	    sum += tree->insert_lengthDistribution[i];
	}
	//fprintf(stderr, "Read Benner & Chang's model of length distribution.\n");
    } else {
	if ( (ldFile=fopen(filename, "rt"))==NULL ) {
	    fprintf(stderr, "Error opening length distribution file: '%s'\n", filename);
	    exit(0);
	}
	i=1; sum=0; P=0;
	while(!feof(ldFile) && i<=tree->maxIndel) {
	    if (fscanf(ldFile,"%lf",&P)!=1) {
		fprintf(stderr,"Unable to read length distribution file: '%s'\n",filename);
		exit(0);
	    }
	    if(action == INSERT) { 
		tree->insert_lengthDistribution[i]=P;
	    	sum+=tree->insert_lengthDistribution[i];
	    } else {
		tree->delete_lengthDistribution[i]=P;
	    	sum+=tree->delete_lengthDistribution[i];
	    }
	    i++; ch=fgetc(ldFile);
	}
	fclose(ldFile);
    }

    /* If sum is not equal to 1, need to normalize it to be 1. */
    if(sum != 1) {
	for(i=1; i<=tree->maxIndel; i++) {
	    if(action==INSERT) tree->insert_lengthDistribution[i] *= (1.0 / sum);
	    else tree->delete_lengthDistribution[i] *= (1.0 / sum);
	}
    }

    /*for(i=1; i<=tree->maxIndel; i++) {
	if(action==INSERT) fprintf(stderr,"Ins Gap size %d:\t %lf\n",i,tree->insert_lengthDistribution[i]);
	else fprintf(stderr,"Del Gap size %d:\t %lf\n",i,tree->delete_lengthDistribution[i]);
    }*/
}
/**/

int IsTreeAvail(FILE *fv)
{
	char ch;

	ch=fgetc(fv);
	while (!feof(fv) && ch!='(' && ch!='[') 
		ch=fgetc(fv);
		
	if (ch=='(' || ch=='[')
		ungetc(ch, fv);
		
	return (!feof(fv));
}


void WriteNode(FILE *fv, TTree *tree, TNode *node)
{
	if (node->tipNo==-1) {
		fputc('(', fv);				
		WriteNode(fv, tree, node->branch1);
		fputc(',', fv);			
		WriteNode(fv, tree, node->branch2);
		fputc(')', fv);				
	} else {
		fprintf(fv, "%s", tree->names[node->tipNo]);
	}
	if (tree->lengths)
		fprintf(fv, ":%.6f", node->length0);
}


void WriteTree(FILE *fv, TTree *tree)
{
	fputc('(', fv);	
	if (tree->rooted) {
		WriteNode(fv, tree, tree->root->branch1);
		fputc(',', fv);				
		WriteNode(fv, tree, tree->root->branch2);
	} else {
		WriteNode(fv, tree, tree->root->branch1);
		fputc(',', fv);				
		WriteNode(fv, tree, tree->root->branch2);
		fputc(',', fv);	
		WriteNode(fv, tree, tree->root->branch0);
	}
	fprintf(fv, ");\n");
}


void UnrootRTree(TTree *tree) 
/* Used to unroot a rooted tree */
{
	TNode *P, *Q, *R, *newRoot;
	double len, len2;
		
	if (!tree->rooted || tree->numTips<3)
		return;
	
	P=tree->tips[0];
	Q=P->branch0; newRoot=Q;
	while (Q!=tree->root) {
		R=Q->branch0;
		len=Q->length0;
		if (Q->branch1==P) {
			len2=Q->length1;
			Q->branch1=R;
			Q->length1=len;
		} else {
			len2=Q->length2;
			Q->branch2=R;
			Q->length2=len;
		}
		Q->branch0=P;
		Q->length0=len2;

		P=Q;
		Q=R;
	}

	len=R->length1+R->length2;
	if (R->branch1==P)
		Q=R->branch2;
	else
		Q=R->branch1;

	Q->branch0=P;
	Q->length0=len;
	
	if (P->branch1==R) {
		P->branch1=Q;
		P->length1=len;
	} else {
		P->branch2=Q;
		P->length2=len;
	}
	
	tree->root=newRoot;
	
	DisposeNode(R);
	
	tree->rooted=0;
}


void RerootUTree(TTree *tree, int tip) 
/* Used to reroot an unrooted tree */
/* This may sound strange but all it does is move
   the root trichotomy  */
{
	TNode *P, *Q, *R, *newRoot;
	double len, len2;
		
	if (tree->rooted)
		return;
	
	P=tree->tips[tip];
	Q=P->branch0; newRoot=Q;
	while (P!=tree->root) {
		R=Q->branch0;
		len=Q->length0;
		if (Q->branch1==P) {
			len2=Q->length1;
			Q->branch1=R;
			Q->length1=len;
		} else {
			len2=Q->length2;
			Q->branch2=R;
			Q->length2=len;
		}
		Q->branch0=P;
		Q->length0=len2;

		P=Q;
		Q=R;
	}
	 	
	tree->root=newRoot;
}
